@@ -172,7 +172,7 @@ static void
 form_layout_changed_cb (GdauiBasicForm *raw_form, GdauiForm *form)
 {
 	gboolean expand;
-	g_object_get (G_OBJECT (form->priv->raw_form), "can-expand", &expand, NULL);
+	g_object_get (G_OBJECT (form->priv->raw_form), "can-expand-v", &expand, NULL);
 	gtk_container_child_set (GTK_CONTAINER (form), form->priv->raw_form,
 				 "expand", expand, "fill", expand, NULL);
 	gtk_widget_queue_resize ((GtkWidget*) form);
@@ -240,7 +240,7 @@ gdaui_form_set_property (GObject *object,
 			gboolean expand;
 			model = GDA_DATA_MODEL (g_value_get_object (value));
 			g_object_set (G_OBJECT (form->priv->raw_form), "model", model, NULL);
-			g_object_get (G_OBJECT (form->priv->raw_form), "can-expand", &expand, NULL);
+			g_object_get (G_OBJECT (form->priv->raw_form), "can-expand-v", &expand, NULL);
 			gtk_container_child_set (GTK_CONTAINER (form), form->priv->raw_form,
 						 "expand", expand, "fill", expand, NULL);
 			gtk_widget_queue_resize ((GtkWidget*) form);
