@@ -628,11 +628,6 @@ action_new_cb (GtkAction *action, GdauiRawForm *form)
 		}
 		else
 			g_object_unref (param);
-
-		GtkWidget *entry;
-		entry = gdaui_basic_form_get_entry_widget (GDAUI_BASIC_FORM (form), (GdaHolder*) list->data);
-		if (entry)
-			gdaui_entry_shell_set_unknown ((GdauiEntryShell*) entry, FALSE);
 	}
 
 	g_signal_handlers_unblock_by_func (form, G_CALLBACK (form_holder_changed_cb), NULL);
