@@ -152,6 +152,7 @@ static void
 gda_meta_struct_init (GdaMetaStruct *mstruct) {
 	mstruct->priv = g_new0 (GdaMetaStructPrivate, 1);
 	mstruct->priv->store = NULL;
+	mstruct->priv->db_objects = NULL;
 	mstruct->priv->index = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 }
 
@@ -311,11 +312,12 @@ compute_view_dependencies (GdaMetaStruct *mstruct, GdaMetaDbObject *view_dbobj,
 				tmp_obj = gda_meta_struct_complement (m2,
 								      GDA_META_DB_VIEW, NULL, NULL, vname, NULL);
 			gda_value_free (vname);
-			g_object_unref (m2);
 
-			if (!tmp_obj) 
+			if (!tmp_obj) {
 				/* could not find dependency */
+				g_object_unref (m2);
 				continue;
+			}
 
 			/* the dependency exists, and is identified by tmp_obj->obj_catalog, tmp_obj->obj_schema 
 			 * and tmp_obj->obj_name */
@@ -336,6 +338,7 @@ compute_view_dependencies (GdaMetaStruct *mstruct, GdaMetaDbObject *view_dbobj,
 						       tmp_obj->obj_schema, tmp_obj->obj_name);
 				g_hash_table_insert (mstruct->priv->index, str, ref_obj);
 			}
+			g_object_unref (m2);
 			g_assert (ref_obj);
 			gda_value_free (catalog);
 			gda_value_free (schema);
@@ -1037,7 +1040,7 @@ _meta_struct_complement (GdaMetaStruct *mstruct, GdaMetaDbObjectType type,
 
  onerror:
 	if (dbo)
-		gda_meta_db_object_free (dbo);
+		dbo->obj_type = GDA_META_DB_UNKNOWN;
 
 	return NULL;
 }
@@ -1787,6 +1790,7 @@ gda_meta_db_object_free_contents (GdaMetaDbObject *dbo)
 		TO_IMPLEMENT;
 	}
 	g_slist_free (dbo->depend_list);
+	memset (dbo, 0, sizeof (GdaMetaDbObject));
 }
 
 static void
