@@ -1,5 +1,5 @@
 /* GDA Capi provider
- * Copyright (C) 2008 - 2009 The GNOME Foundation.
+ * Copyright (C) 2008 - 2010 The GNOME Foundation.
  *
  * AUTHORS:
  *      TO_ADD: your name and email
@@ -111,6 +111,9 @@ static GObject             *gda_capi_provider_statement_execute (GdaServerProvid
 								 GType *col_types, GdaSet **last_inserted_row, 
 								 guint *task_id, GdaServerProviderExecCallback async_cb, 
 								 gpointer cb_data, GError **error);
+static GdaSqlStatement     *gda_capi_statement_rewrite          (GdaServerProvider *provider, GdaConnection *cnc,
+								 GdaStatement *stmt, GdaSet *params, GError **error);
+
 
 /* distributed transactions */
 static gboolean gda_capi_provider_xa_start    (GdaServerProvider *provider, GdaConnection *cnc, 
@@ -191,6 +194,7 @@ gda_capi_provider_class_init (GdaCapiProviderClass *klass)
 						  * not call calls gda_statement_to_sql_extended() */
 	provider_class->statement_prepare = gda_capi_provider_statement_prepare;
 	provider_class->statement_execute = gda_capi_provider_statement_execute;
+	provider_class->statement_rewrite = gda_capi_statement_rewrite;
 
 	provider_class->is_busy = NULL;
 	provider_class->cancel = NULL;
@@ -1072,7 +1076,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 		
 		/* find requested parameter */
 		if (!params) {
-			event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+			event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
 			gda_connection_event_set_description (event, _("Missing parameter(s) to execute query"));
 			g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 				     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR,
@@ -1092,7 +1096,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			if (! allow_noparam) {
 				gchar *str;
 				str = g_strdup_printf (_("Missing parameter '%s' to execute query"), pname);
-				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
 				gda_connection_event_set_description (event, str);
 				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
@@ -1111,7 +1115,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 			if (! allow_noparam) {
 				gchar *str;
 				str = g_strdup_printf (_("Parameter '%s' is invalid"), pname);
-				event = gda_connection_event_new (GDA_CONNECTION_EVENT_ERROR);
+				event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_ERROR);
 				gda_connection_event_set_description (event, str);
 				g_set_error (error, GDA_SERVER_PROVIDER_ERROR,
 					     GDA_SERVER_PROVIDER_MISSING_PARAM_ERROR, "%s", str);
@@ -1125,6 +1129,39 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
                                 continue;
                         }
 		}
+		else if (gda_holder_value_is_default (h) && !gda_holder_get_value (h)) {
+			/* create a new GdaStatement to handle all default values and execute it instead
+			 * needs to be adapted to take into account how the database server handles default
+			 * values (some accept the DEFAULT keyword), changing the 3rd argument of the
+			 * gda_statement_rewrite_for_default_values() call
+			 */
+			GdaSqlStatement *sqlst;
+			GError *lerror = NULL;
+			sqlst = gda_statement_rewrite_for_default_values (stmt, params, TRUE, &lerror);
+			if (!sqlst) {
+				event = gda_connection_point_available_event (cnc,
+									      GDA_CONNECTION_EVENT_ERROR);
+				gda_connection_event_set_description (event, lerror && lerror->message ? 
+								      lerror->message :
+								      _("Can't rewrite statement handle default values"));
+				g_propagate_error (error, lerror);
+				break;
+			}
+			
+			GdaStatement *rstmt;
+			GObject *res;
+			rstmt = g_object_new (GDA_TYPE_STATEMENT, "structure", sqlst, NULL);
+			gda_sql_statement_free (sqlst);
+			res = gda_capi_provider_statement_execute (provider, cnc,
+								   rstmt, params,
+								   model_usage,
+								   col_types, last_inserted_row,
+								   task_id,
+								   async_cb, cb_data, error);
+			g_object_unref (rstmt);
+			return res;
+		}
+
 
 		/* actual binding using the C API, for parameter at position @i */
 		const GValue *value = gda_holder_get_value (h);
@@ -1138,7 +1175,7 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 	}
 	
 	/* add a connection event for the execution */
-	event = gda_connection_event_new (GDA_CONNECTION_EVENT_COMMAND);
+	event = gda_connection_point_available_event (cnc, GDA_CONNECTION_EVENT_COMMAND);
         gda_connection_event_set_description (event, _GDA_PSTMT (ps)->sql);
         gda_connection_add_event (cnc, event);
 
@@ -1201,6 +1238,23 @@ gda_capi_provider_statement_execute (GdaServerProvider *provider, GdaConnection
 	}
 }
 
+/*
+ * Rewrites a statement in case some parameters in @params are set to DEFAULT, for INSERT or UPDATE statements
+ *
+ * Usually it uses the DEFAULT keyword or removes any default value inserted or updated, see
+ * gda_statement_rewrite_for_default_values()
+ */
+static GdaSqlStatement *
+gda_capi_statement_rewrite (GdaServerProvider *provider, GdaConnection *cnc,
+			    GdaStatement *stmt, GdaSet *params, GError **error)
+{
+	if (cnc) {
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+		g_return_val_if_fail (gda_connection_get_provider (cnc) == provider, NULL);
+	}
+	return gda_statement_rewrite_for_default_values (stmt, params, TRUE, error);
+}
+
 /*
  * starts a distributed transaction: put the XA transaction in the ACTIVE state
  */
