@@ -26,6 +26,7 @@
 #include "browser-core.h"
 #include "browser-window.h"
 #include "login-dialog.h"
+#include <libgda-ui/internal/utility.h>
 
 #ifdef HAVE_MAC_INTEGRATION
 #include <gtkosxapplication.h>
@@ -122,7 +123,8 @@ browser_connection_close (GtkWindow *parent, BrowserConnection *bcnc)
  * @format: printf() style format string
  * @...: arguments for @format
  *
- * Displays a modal error until the user aknowledges it.
+ * Displays an error message until the user aknowledges it. I @parent is a #BrowserWindow, then
+ * the error message is displayed in the window if possible
  */
 void
 browser_show_error (GtkWindow *parent, const gchar *format, ...)
@@ -137,6 +139,12 @@ browser_show_error (GtkWindow *parent, const gchar *format, ...)
         vsnprintf (sz, sizeof (sz), format, args);
         va_end (args);
 
+	if (BROWSER_IS_WINDOW (parent)) {
+		browser_window_show_notice (BROWSER_WINDOW (parent), GTK_MESSAGE_ERROR,
+					    NULL, sz);
+		return;
+	}
+
         /* create the error message dialog */
 	dialog = gtk_message_dialog_new (parent,
 					 GTK_DIALOG_DESTROY_WITH_PARENT |
@@ -151,90 +159,6 @@ browser_show_error (GtkWindow *parent, const gchar *format, ...)
         gtk_widget_destroy (dialog);
 }
 
-/* hash table to remain which context notices have to be hidden: key=context, value=GINT_TO_POINTER (1) */
-static GHashTable *hidden_contexts = NULL;
-
-static void
-hide_notice_toggled_cb (GtkToggleButton *toggle, gchar *context)
-{
-	g_assert (hidden_contexts);
-	if (gtk_toggle_button_get_active (toggle))
-		g_hash_table_insert (hidden_contexts, g_strdup (context), GINT_TO_POINTER (TRUE));
-	else
-		g_hash_table_remove (hidden_contexts, context);
-}
-
-/**
- * browser_show_notice
- * @parent: a #GtkWindow
- * @context: a context string or %NULL
- * @format: printf() style format string
- * @...: arguments for @format
- *
- * Displays a modal notice until the user aknowledges it.
- *
- * If @context is not NULL, then the message box contains a check box to avoid displaying the
- * same massage again.
- */
-void
-browser_show_notice (GtkWindow *parent, const gchar *context, const gchar *format, ...)
-{
-        va_list args;
-        gchar sz[2048];
-        GtkWidget *dialog;
-	gboolean hide = FALSE;
-
-        /* build the message string */
-        va_start (args, format);
-        vsnprintf (sz, sizeof (sz), format, args);
-        va_end (args);
-
-	if (context) {
-		if (!hidden_contexts)
-			hidden_contexts = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-		hide = GPOINTER_TO_INT (g_hash_table_lookup (hidden_contexts, context));
-	}
-
-	if (hide) {
-		if (BROWSER_IS_WINDOW (parent)) {
-			gchar *ptr;
-			for (ptr = sz; *ptr && (*ptr != '\n'); ptr++);
-			if (*ptr) {
-				*ptr = '.'; ptr++;
-				*ptr = '.'; ptr++;
-				*ptr = '.'; ptr++;
-				*ptr = 0;
-			}
-			browser_window_push_status (BROWSER_WINDOW (parent),
-						    "SupportNotice", sz, TRUE);
-		}
-	}
-	else {
-		/* create the error message dialog */
-		dialog = gtk_message_dialog_new_with_markup (parent,
-							     GTK_DIALOG_DESTROY_WITH_PARENT |
-							     GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
-							     GTK_BUTTONS_CLOSE,
-							     "<span weight=\"bold\">%s</span>\n%s", _("Note:"), sz);
-		if (context) {
-			GtkWidget *cb;
-			cb = gtk_check_button_new_with_label (_("Don't show this message again"));
-			g_signal_connect_data (cb, "toggled",
-					       G_CALLBACK (hide_notice_toggled_cb), g_strdup (context),
-					       (GClosureNotify) g_free, 0);
-#if GTK_CHECK_VERSION(2,18,0)
-			gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), cb, FALSE, FALSE, 10);
-#else
-			gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), cb, FALSE, FALSE, 10);
-#endif
-		}
-		
-		gtk_widget_show_all (dialog);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
-	}
-}
-
 #ifdef HAVE_GDU
 /**
  * browser_show_help
@@ -314,7 +238,8 @@ browser_show_help (GtkWindow *parent, const gchar *topic)
 		g_error_free (error);
 	}
 	else
-		browser_show_notice (parent, "show-help", _("Help is being loaded, please wait..."));
+		browser_window_show_notice (BROWSER_WINDOW (parent), GTK_MESSAGE_INFO,
+					    "show-help", _("Help is being loaded, please wait..."));
 
 	g_free (uri);
 }
@@ -425,6 +350,7 @@ browser_get_pixbuf_icon (BrowserIconType type)
 		"gda-browser-query.png",
 		"gda-browser-grid.png",
 		"gda-browser-form.png",
+		"gda-browser-menu-ind.png",
 	};
 
 	if (!array)
@@ -583,30 +509,6 @@ browser_make_small_button (gboolean is_toggle, const gchar *label, const gchar *
 	return button;
 }
 
-static gboolean
-tree_view_button_pressed_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
-{
-	GtkTreeView *tree_view;
-	GtkTreeSelection *selection;
-
-	if (event->button != 3)
-		return FALSE;
-
-	tree_view = GTK_TREE_VIEW (widget);
-	selection = gtk_tree_view_get_selection (tree_view);
-
-	/* force selection of row on which clicked occurred */
-	GtkTreePath *path;
-	if ((event->window == gtk_tree_view_get_bin_window (tree_view)) &&
-	    gtk_tree_view_get_path_at_pos (tree_view, event->x, event->y, &path, NULL, NULL, NULL)) {
-		gtk_tree_selection_unselect_all (selection);
-		gtk_tree_selection_select_path (selection, path);
-		gtk_tree_path_free (path);
-	}
-
-	return FALSE;
-}
-
 /**
  * browser_make_tree_view
  * @model: a #GtkTreeModel
@@ -623,8 +525,6 @@ browser_make_tree_view (GtkTreeModel *model)
 	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), NULL);
 	tv = gtk_tree_view_new_with_model (model);
 
-	g_signal_connect (G_OBJECT (tv), "button-press-event",
-                          G_CALLBACK (tree_view_button_pressed_cb), NULL);
-
+	_gdaui_setup_right_click_selection_on_treeview (GTK_TREE_VIEW (tv));
 	return tv;
 }
